#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Wetter
#
# This small class will show the local weather in Vienna, by "parsing" a
# remote website that contains this information.
#
# The main regex in use can also be anchored towards "12.2&deg;", if
# otherwise the dataset is nil.
#
# Usage example from ruby:
#
#   Wetter.new
#
# =========================================================================== #
# require 'wetter/wetter.rb'
# =========================================================================== #
require 'wetter/requires/external_requires.rb'
require 'wetter/constants.rb'
require 'wetter/menu.rb'
require 'wetter/toplevel_methods/is_on_roebe.rb'

class Wetter # === Wetter

  include Colours
  include Colours::E

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_commandline_arguments = ARGV,
      run_already                    = true
    )
    reset
    if block_given?
      yielded = yield
      case yielded
      when :be_silent
        @be_verbose = false
      end
    end
    if optional_commandline_arguments
      menu(optional_commandline_arguments)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    @debug = false
    @be_verbose = true
  end

  # ========================================================================= #
  # === enable_debug_mode
  # ========================================================================= #
  def enable_debug_mode
    @debug = true
  end

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @debug
  end

  # ========================================================================= #
  # === cliner
  # ========================================================================= #
  def cliner
    '=' * 80
  end

  # ========================================================================= #
  # === sanitize_dataset
  # ========================================================================= #
  def sanitize_dataset
    splitted = @data.split(N)
    splitted.reject! {|line|
      ! (line.include?('title="Wien Innere Stadt') and line.include?('Temp: '))
    }
    splitted = splitted.first if splitted.is_a? Array
    if debug?
      opn; e 'Now showing the sanitized dataset, as we run in @debug mode.'
      pp splitted
    end
    @data = splitted
    if @data.nil?
      @data =~ USE_DEGREE_REGEX
      unless $1.to_s.dup.empty?
        @data = $1.to_s.dup
      end
    end
  end

  # ========================================================================= #
  # === find_match
  # ========================================================================= #
  def find_match
    @data  =~ USE_THIS_REGEX
    @match = $1.to_s.dup
  end

  # ========================================================================= #
  # === match?
  # ========================================================================= #
  def match?
    @match
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e
    e '  --URL # show the remote URL'
    e
  end

  # ========================================================================= #
  # === show_the_remote_url
  #
  # To invoke this method, do:
  #
  #   wetter --URL
  #
  # ========================================================================= #
  def show_the_remote_url
    e
    e sfancy("  #{URL}")
    e
  end

  # ========================================================================= #
  # === try_to_make_use_of_the_unicode_weather_symbol?
  # ========================================================================= #
  def try_to_make_use_of_the_unicode_weather_symbol?
    TRY_TO_MAKE_USE_OF_THE_UNICODE_WEATHER_SYMBOL
  end

  # ========================================================================= #
  # === report_result
  # ========================================================================= #
  def report_result
    if @be_verbose
      n_degrees_celsius = match?.to_f
      result = "The current temperature in Vienna is: "\
               "#{sfancy(n_degrees_celsius)} °C".dup
      if try_to_make_use_of_the_unicode_weather_symbol?
        begin
          require 'roebe/toplevel_methods/unicode/popular_unicode_symbols.rb'
          case n_degrees_celsius
          when 16..50 # Should be clear weather most of the time.
            result << " #{tomato(Roebe.clear_weather)}"
          when 5..15 # Medium-temperature; usually cloudy.
            result << " #{royalblue(Roebe.cloudy_weather_symbol)}"
          when -30..4 # It's really quite cold.
            result << " #{royalblue(Roebe.snowman)}"
          end
        rescue LoadError
        end
      end
      opn; e result
    end
  end

  # ========================================================================= #
  # === royalblue
  # ========================================================================= #
  def royalblue(i = '')
    Colours.royalblue(i)
  end

  # ========================================================================= #
  # === tomato
  # ========================================================================= #
  def tomato(i = '')
    Colours.tomato(i)
  end

  # ========================================================================= #
  # === read_in_dataset_from_remote_webpage
  # ========================================================================= #
  def read_in_dataset_from_remote_webpage
    @data = URI.open(URL).read
    if debug?
      opn; e 'Now showing @data, as we run in @debug mode.'
      e cliner
      e @data
      e cliner
    end
    return @data
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    read_in_dataset_from_remote_webpage
    sanitize_dataset
    find_match
    report_result
  end

  # ========================================================================= #
  # === Wetter.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    self.new(i)
  end

  # ========================================================================= #
  # === Wetter.current_degrees
  #
  # This method will simply return a Float number, the current temperature
  # in Vienna by default.
  # ========================================================================= #
  def self.current_degrees
    Wetter.new { :be_silent }.match?.to_f
  end; self.instance_eval { alias n_degrees_celsius? current_degrees } # === Wetter.n_degrees_celsius?
       self.instance_eval { alias temperature?       current_degrees } # === Wetter.temperature?

end

if __FILE__ == $PROGRAM_NAME
  Wetter.new(ARGV)
end # wetter --debug